SystemOrganization addCategory: #ConfigurationOfNacl!

Object subclass: #ConfigurationOfNacl
	instanceVariableNames: 'project library'
	classVariableNames: 'LastVersionLoad'
	poolDictionaries: ''
	category: 'ConfigurationOfNacl'!

!ConfigurationOfNacl commentStamp: '<historical>' prior: 0!
#Overview
- This is a Pharo/Sqeuak binding to libsodium, a portable shared-library version of the NaCl cryptography library. The library name, NaCl, is pronounced "salt" and stands for Networking and Cryptography Library.
- The primary goal of NaCl is to cryptographically protect every network connection, providing strong condentiality, strong integrity, and state-of-the-art availability against attackers sning or modifying network packets.
- See paper at http://cr.yp.to/highspeed/coolnacl-20120725.pdf for details'
- Can be loaded from ConfigurationBrowser within Pharo

# Examples

## Hashing with SHA-512

	(Nacl hashString: ''The quick brown fox jumps over the lazy dog'') hex 

## Encryption

	| senderSecretKey recipientPublicKey nonce cryptoBox envelope |

	senderSecretKey := ByteArray readHexFrom: ''d54ff4b666a43070ab20937a92c49ecf65503583f8942350fc197c5023b015c3''.
	recipientPublicKey := ByteArray readHexFrom: ''de1042928b74e9f96cf3f3e290c16cb4eba9c696e9a1e15c7f4d0514ddce1154''.
	nonce := ByteArray readHexFrom: ''065114ca5a687e0544a88e6fc757b30afc70a0355854fd54''.
	cryptoBox := NaclBoxSecretKey skBytes: senderSecretKey 	pkBytes: recipientPublicKey.
	envelope := cryptoBox 
		box: ''hello'' asUtf8ByteArray
		nonce: nonce
		publicKey: cryptoBox publicKey.
!

!ConfigurationOfNacl class methodsFor: 'development support' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
DevelopmentSupport

"See the methods in the 'development support' category on the class-side of MetacelloBaseConfiguration. Decide what development support methods you would like to use and copy them the the class-side of your configuration."
	<apiDocumentation>
! !

!ConfigurationOfNacl class methodsFor: 'private' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
baseConfigurationClassIfAbsent: aBlock

	^Smalltalk
		at: #MetacelloBaseConfiguration
		ifAbsent: [ 
			self ensureMetacelloBaseConfiguration.
			Smalltalk at: #MetacelloBaseConfiguration ifAbsent: aBlock ].
! !

!ConfigurationOfNacl class methodsFor: 'catalog' stamp: 'HernanMoralesDurand 1/22/2014 06:35'!
catalogChangeLog
		
	^ ''

! !

!ConfigurationOfNacl class methodsFor: 'catalog' stamp: 'HernanMoralesDurand 1/22/2014 21:20'!
catalogContactInfo
	^ 'Configuration by hernan.morales@gmail.com
Binding developement by tonygarnockjones@gmail.com'! !

!ConfigurationOfNacl class methodsFor: 'catalog' stamp: 'HernanMoralesDurand 1/22/2014 22:12'!
catalogDescription

	^ 'This is a Pharo/Sqeuak binding to libsodium, a portable shared-library version of the NaCl cryptography library. The library name, NaCl, is pronounced "salt" and stands for Networking and Cryptography Library.
	
The primary goal of NaCl is to cryptographically protect every network connection, providing strong condentiality, strong integrity, and state-of-the-art availability against attackers sning or modifying network packets.

!!!! Features

- Authenticated public-key and authenticated shared-key encryption
- Public-key and shared-key signatures
- Hashing
- Keyed hashes for short messages
- Secure pseudo-random numbers generation

See paper at http://cr.yp.to/highspeed/coolnacl-20120725.pdf for details'! !

!ConfigurationOfNacl class methodsFor: 'catalog' stamp: 'HernanMoralesDurand 1/22/2014 06:35'!
catalogKeyClassesAndExample

	^ ' ==Nacl== is the main public class of the package. It is the class to interact with as shown by the tests. 

Example of hashing

[[[
(Nacl hashString: ''The quick brown fox jumps over the lazy dog'') hex 
]]]

The following is an example of encryption:
[[[ 
| senderSecretKey recipientPublicKey nonce cryptoBox envelope |

senderSecretKey := ByteArray readHexFrom: ''d54ff4b666a43070ab20937a92c49ecf65503583f8942350fc197c5023b015c3''.
recipientPublicKey := ByteArray readHexFrom: ''de1042928b74e9f96cf3f3e290c16cb4eba9c696e9a1e15c7f4d0514ddce1154''.
nonce := ByteArray readHexFrom: ''065114ca5a687e0544a88e6fc757b30afc70a0355854fd54''.
cryptoBox := NaclBoxSecretKey skBytes: senderSecretKey pkBytes: recipientPublicKey.
envelope := cryptoBox 
	box: ''hello'' asUtf8ByteArray
	nonce: nonce
	publicKey: cryptoBox publicKey.
]]]

'! !

!ConfigurationOfNacl class methodsFor: 'catalog' stamp: 'HernanMoralesDurand 1/22/2014 06:28'!
catalogKeywords
	^ #(security nacl encryption decryption salt hashing libsodium crypto sha)! !

!ConfigurationOfNacl class methodsFor: 'private' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
ensureMetacello

	(self baseConfigurationClassIfAbsent: []) ensureMetacello! !

!ConfigurationOfNacl class methodsFor: 'private' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
ensureMetacelloBaseConfiguration

	Smalltalk
		at: #MetacelloBaseConfiguration
		ifAbsent: [ 
			| repository version |
			repository := MCHttpRepository location: 'http://seaside.gemstone.com/ss/metacello' user: '' password: ''.
			repository
				versionReaderForFileNamed: 'Metacello-Base-DaleHenrichs.2.mcz'
				do: [ :reader | 
					version := reader version.
					version load.
					version workingCopy repositoryGroup addRepository: repository ] ]! !

!ConfigurationOfNacl class methodsFor: 'metacello tool support' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
isMetacelloConfig
	"Answer true and the Metacello tools will operate on you"
	
	^true! !

!ConfigurationOfNacl class methodsFor: 'loading' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
load
	"Load the #stable version defined for this platform. The #stable version is the version that is recommended to be used on this platform."

	"self load"

	<apiDocumentation>
	^(self project version: #stable) load! !

!ConfigurationOfNacl class methodsFor: 'loading' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
loadBleedingEdge
	"Load the latest versions of the mcz files defined for this project. It is not likely that the #bleedingEdge has been tested."

	"self loadBleedingEdge"

	<apiDocumentation>
	^(self project version: #bleedingEdge) load! !

!ConfigurationOfNacl class methodsFor: 'loading' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
loadDevelopment
	"Load the #development version defined for this platform. The #development version will change over time and is not expected to be stable."

	"self loadDevelopment"

	<apiDocumentation>
	^(self project version: #development) load! !

!ConfigurationOfNacl class methodsFor: 'accessing' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
project

	^self new project! !

!ConfigurationOfNacl class methodsFor: 'development support' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
validate
	"Check the configuration for Errors, Critical Warnings, and Warnings (see class comment for MetacelloMCVersionValidator for more information). 
	Errors identify specification issues that will result in unexpected behaviour when you load the configuration. 
	Critical Warnings identify specification issues that may result in unexpected behavior when you load the configuration.
	Warnings identify specification issues that are technically correct, but are worth take a look at."

	"self validate"

	<apiDocumentation>
	self ensureMetacello.
	^ ((Smalltalk at: #MetacelloToolBox) validateConfiguration: self debug: #() recurse: false) explore! !

!ConfigurationOfNacl methodsFor: 'baselines' stamp: 'HernanMoralesDurand 1/24/2014 02:17'!
baseline10: spec
    <version: '1.0-baseline'>
    
	spec for: #common do: [
		spec blessing: #baseline.
		spec description: 'First release'.
		spec preLoadDoIt: #preLoad.
		spec postLoadDoIt: #postLoad.
		spec repository: 'http://www.smalltalkhub.com/mc/tonyg/Crypto-Nacl/main'.
		
		spec package: 'Crypto-Nacl' with: [ spec requires: #('OSProcess' 'FFI') ].
		spec group: 'default' with: #('Crypto-Nacl' ) ].
	
	spec for: #'pharo3.x' do: [ 
		self specForOSProcessPharo3: spec.		
		self specForFFIPharo3: spec ].
	spec for: #'pharo2.x' do: [ 
		self specForOSProcessPharo2: spec.
		self specForFFIPharo2: spec ].
        ! !

!ConfigurationOfNacl methodsFor: 'accessing' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
customProjectAttributes
	 "Edit to return a collection of any custom attributes e.g. for conditional loading: Array with: #'Condition1' with: #'Condition2.
	For more information see: http://code.google.com/p/metacello/wiki/CustomProjectAttrributes"

	^ #().! !

!ConfigurationOfNacl methodsFor: 'symbolic versions' stamp: 'HernanMoralesDurand 1/22/2014 06:25'!
development: spec
	<symbolicVersion: #'development'>

	spec for: #'common' version: '1.0'.
! !

!ConfigurationOfNacl methodsFor: 'doits' stamp: 'HernanMoralesDurand 1/24/2014 02:40'!
platformLibraryUrl
	" Answer a download URL for the compiled version of library "

	Smalltalk os isWin32 
		ifTrue: [ ^ 'http://dl.dropboxusercontent.com/u/103833630/libsodium.zip' ].
	Smalltalk os isUnix 
		ifTrue: [ ^ 'https://dl.dropboxusercontent.com/u/103833630/libsodium.so.gz' ].
	Smalltalk os isMac 
		ifTrue: [ ^ 'https://dl.dropboxusercontent.com/u/103833630/libsodium.dylib.gz' ].
	self error: 'Cannot find library for your OS platform'.! !

!ConfigurationOfNacl methodsFor: 'doits' stamp: 'HernanMoralesDurand 1/24/2014 02:10'!
postLoad
	" chmod library file "

	Smalltalk os isUnix 
		ifTrue: [ 
			| fileName |
			fileName := (FileSystem disk workingDirectory / 'pharo-vm' / 'libsodium') fullName.
			(Smalltalk at: #OSProcess) command: 'chmod 755 ' , fileName ]		
			! !

!ConfigurationOfNacl methodsFor: 'doits' stamp: 'HernanMoralesDurand 1/23/2014 00:48'!
preLoad

	| url fileRef response |

	url := self platformLibraryUrl asZnUrl.
	fileRef := FileSystem disk workingDirectory / url pathSegments last.
	(response := ZnEasy get: url) isSuccess
		ifTrue: [ fileRef writeStreamDo: [ : stream | stream nextPutAll: response contents ] ]
		ifFalse: [ self error: 'Cannot download compiled library file' ].
	self uncompress: fileRef.! !

!ConfigurationOfNacl methodsFor: 'accessing' stamp: 'HernanMoralesDurand 1/22/2014 05:05'!
project

	^ project ifNil: [ | constructor |
		"Bootstrap Metacello if it is not already loaded"
		(self class baseConfigurationClassIfAbsent: []) ensureMetacello.
		"Construct Metacello project"
		project := MetacelloMCProject new projectAttributes: self customProjectAttributes. 
		constructor := (Smalltalk at: #MetacelloVersionConstructor) on: self project: project.
		project loadType: #linear. "change to #atomic if desired"
		project ]! !

!ConfigurationOfNacl methodsFor: 'baselines' stamp: 'HernanMoralesDurand 1/23/2014 03:47'!
specForFFIPharo2: spec

	spec
		project: 'FFI' with: [
			spec
				className: 'ConfigurationOfFFI';
				version: #stable;
				repository: 'http://ss3.gemtalksystems.com/ss/MetaRepoForPharo20' ];
		yourself.
        ! !

!ConfigurationOfNacl methodsFor: 'baselines' stamp: 'HernanMoralesDurand 1/23/2014 03:46'!
specForFFIPharo3: spec

	spec
		project: 'FFI' with: [
			spec
				className: 'ConfigurationOfFFI';
				version: #stable;
				repository: 'http://www.smalltalkhub.com/mc/Pharo/MetaRepoForPharo30/main' ];
		yourself.
        ! !

!ConfigurationOfNacl methodsFor: 'baselines' stamp: 'HernanMoralesDurand 1/24/2014 01:58'!
specForOSProcessPharo2: spec

	spec project: 'OSProcess' with: [
		spec
			className: 'ConfigurationOfOSProcess';
			version: #stable;
			repository: 'http://ss3.gemtalksystems.com/ss/MetaRepoForPharo20' ];
		yourself
! !

!ConfigurationOfNacl methodsFor: 'baselines' stamp: 'HernanMoralesDurand 1/24/2014 01:58'!
specForOSProcessPharo3: spec

	spec project: 'OSProcess' with: [
		spec
			className: 'ConfigurationOfOSProcess';
			version: #stable;
			repository: 'http://www.smalltalkhub.com/mc/Pharo/MetaRepoForPharo30/main' ];
		yourself
! !

!ConfigurationOfNacl methodsFor: 'symbolic versions' stamp: 'HernanMoralesDurand 1/23/2014 04:06'!
stable: spec
	<symbolicVersion: #'stable'>

	spec for: #'common' version: '1.0'.
	spec for: #'pharo3.x' version: '1.0'.
	spec for: #'pharo2.x' version: '1.0'.
! !

!ConfigurationOfNacl methodsFor: 'doits' stamp: 'HernanMoralesDurand 1/23/2014 14:59'!
uncompress: fileRef
	" Private - Uncompress fileRef to the current working directory "

	| dst |
	
	dst := FileSystem workingDirectory / 'pharo-vm'.
	(fileRef fullName endsWith: '.zip')
		ifTrue: [ ZipArchive new 
					readFrom: fileRef fullName;
					extractAllTo: dst ].
	(fileRef fullName endsWith: '.gz')
		ifTrue: [ | unzipped |
			unzipped := GZipReadStream unzip: fileRef basename.
			(FileSystem workingDirectory / unzipped) 
				renameTo: unzipped asFileReference basenameWithoutExtension ].
! !

!ConfigurationOfNacl methodsFor: 'versions' stamp: 'HernanMoralesDurand 1/22/2014 06:22'!
version10: spec
    "The name for this version is solely defined by the following pragma:"
    <version: '1.0' imports: #('1.0-baseline') >
    "Baselines are used to define more complex setups for your project.
    If you want to use external projects and have fine-graind control of the dependencies between packages use the #imports: part.
    See the #baseline10: for more details."
    "Using #for:do: with the #common release specifier the following version declaration is valid for all platforms."
    spec for: #common do: [
        spec
            blessing: #release;       
            description: 'Version 1.0 the current stable release';
            author: 'Hernan Morales Durand';
            timestamp: '2013-22-01'.
        spec package: 'Nacl' with: 'Crypto-Nacl-tonyg.8' ].

! !
